<?php
/*******************************************
File:         spformproc.inc.php

Snippet:       SPForm
Version: 1.0.4
$Revision: 265 $
$Author: Bob Ray $
$Date: 2009-07-02 15:42:55 -0500 (Thu, 02 Jul 2009) $
Adapted from:  Many sources but particularly scform by James Seymour and CFFormProtect by Jake Munson
Compatibility: MODX 0.9.6, 0.9.7

*******************************************/
/*
 * scformproc.php - Process results of scform.php web form
 * Copyright (C) 2002-2004 by James S. Seymour (jseymour [at] LinxNet [dot] com)
 * (See "License", below.)  Release 1.2.5.
 *
 * Features
 *
 *  . Single simple, secure, straight-forward script
 *  . Flexible configuration
 *  . Contact list email addresses are completely hidden, thus
 *    foiling web-page-scraping, scum-sucking, dirtbag spammers
 *      . Easy to add new recipients to the list with just a
 *        text editor
 *      . Uses format similar to common "aliases" style recipient
 *        email address lists
 *      . Contact list shared between web form and form processing
 *  . Optional check for banned email addresses and remote hosts
 *      . Ban list easily edited with common text editor
 *      . Option for mis-direction that allows banned senders
 *        to *think* they're getting through (heh heh heh)
 *
 * TBD
 *    . Add DTD and content/char encoding to error message and
 *      response pages?
 *
 * License:
 *    This program is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU General Public License
 *    as published by the Free Software Foundation; either version 2
 *    of the License, or (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You may have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 *    USA.
 *
 *    An on-line copy of the GNU General Public License can be found
 *    http://www.fsf.org/copyleft/gpl.html.
 *
 * The SCForm Home Page is at:
 *
 *    http://jimsun.LinxNet.com/SCForm.html
 *
 */

// get our config file
 global $modx;
 $spformPath = "assets/snippets/spform/";
 require_once $modx->config['base_path'].$spformPath."spfconfig.cfg.php";
 require_once $modx->config['base_path'].$spformPath."class.phpmailer.php";

 $lang = isset($lang)?$lang:'en';
 require_once $modx->config['base_path'].$spformPath."lang/".$lang.".inc.php";

/*
 * Shouldn't have to touch anything beyond here
 */


$errors = array();

/* Function, shared by the show_errors() and show_fatal() functions,
   to emit the list of problems.  (Note: Does so as an HTML unnumbered
   list.  Assumes caller has already done the HTML "opening" stuff.) */

function show_error_list($errors) {
    $nerrors = count($errors);

    print("<ul>\n");
    for($i = 0; $i < $nerrors; $i++)
    print("<li>" . $errors[$i] . "</li>\n");
    print("</ul>\n");
}

// Show list of non-fatal errors
function show_errors($errors,$spf_lang) {


    echo $spf_lang['form-errors'];

    show_error_list($errors);
}

// Show fatal error
function show_fatal($errors,$spf_lang) {

    echo '<p>'.$spf_lang['fatal1'].'</p>'; //\n");
    show_error_list($errors);
    echo $spf_lang['fatal2'];

}

// Read a line from a config file, stripping comments and blank lines
function read_file_line($fp) {
    while(($inString = fgets($fp, 2048)) != false) {
    $inString = rtrim(preg_replace('/\s*#.*/', '', $inString));
    if(!empty($inString))
        break;
    }

    return $inString;
}

// Function to check the referer against the list of acceptable
// refererers.

//
// If the $referers array is empty, returns true/pass by default
//
function check_referer($referers, $logOnReferer, $spf_lang) {

    global $errors, $scriptName, $formProcBlankRefOkay;

    $found = true;    // Default to "pass"

    // Check only if the array of allowed referers is non-empty or if the HTTP
    // REFERER is empty and we're allowing that
    if(!(empty($_SERVER['HTTP_REFERER']) && $formProcBlankRefOkay)) {
    if(count($referers)) {
        $found = false;

        if(!empty($_SERVER['HTTP_REFERER'])) {
        list($referer) =
            array_slice(explode("/", $_SERVER['HTTP_REFERER']), 2, 1);
        for($x = 0; $x < count($referers); ++$x) {
            if(eregi($referers[$x], $referer)) {
            $found = true;
            break;
            }
        }
        }

        if(!$found) {
        $errors[] = $spf_lang['unauthorized-server'] ."(" .
                $_SERVER['HTTP_REFERER'] . ")";
        if($logOnReferer) {
            error_log("[$scriptName] Illegal Referer. (" .
                   $_SERVER['HTTP_REFERER'] . ") ", 0);
        }
        }
    }
    }

    return $found;
}

// Function to check an IP address match
// Handles CIDR notation in the thing to check against
// Note: Address to check against is expected to be in regexp format
// (i.e.: "."s escaped with "\"s)
function check_ip($chkAgainst, $chkAddr) {

    $addrMatch = false;    // Assume no match

    // If the "check against" value contains a "/", it'll be an IP
    // address (range) in CIDR notation.
    if(ereg('/[0-9]', $chkAgainst)) {

    # Break down dot.ted.qu.ad/bits of address to check against
    list($addrBase, $hostBits) = explode('/', $chkAgainst);
    list($w, $x, $y, $z) = explode('.', $addrBase);

    # Convert to high and low address ints
    $chkAgainst = ($w << 24) + ($x << 16) + ($y << 8) + $z;
    $mask = $hostBits == 0? 0 : (~0 << (32 - $hostBits));
    $lowLimit = $chkAgainst & $mask;
    $highLimit = $chkAgainst | (~$mask & 0xffffffff);

    # Convert addr to check to int
    list($w, $x, $y, $z) = explode('.', $chkAddr);
    $chkAddr = ($w << 24) + ($x << 16) + ($y << 8) + $z;

    $addrMatch = (($chkAddr >= $lowLimit) && ($chkAddr <= $highLimit));

    } else {
    $addrMatch = ereg("^$chkAgainst", $chkAddr);
    }

    return $addrMatch;
}

// Function to check the given email address, the remote host (if
// available) and the remote IP against the ban list.
function check_banlist($logOnBan, $email, $banListFile) {

    global $errors, $scriptName;

    $notAllowed = false;    // Default to allowed

    // "whoto" to email address hash
    $banList = array();

    // Get the banList
    if($fp = @fopen($banListFile, "r")) {
    while($inString = read_file_line($fp))
        $banList[] = $inString;
    fclose($fp);
    }

    if(count($banList)) {
        $emailFix = trim(strtolower($email));
        $remoteHostFix = trim(strtolower($_SERVER['REMOTE_HOST']));

        foreach($banList as $banned) {
            $banFix = trim(strtolower(ereg_replace('\.', '\\.', $banned)));
            if(strstr($banFix, "@")) {            // email address?
            if(ereg('^@', $banFix)) {        // Any user @host?
                // Expand the match expression to catch hosts and
                // sub-domains
                $banFix = ereg_replace('^@', '[@\\.]', $banFix);
                if(($notAllowed = ereg("$banFix$", $emailFix))) {
                $bannedOn = $emailFix;
                break;
                }
            } elseif(ereg('@$', $banFix)) {    // User at any host?
                if(($notAllowed = ereg("^$banFix", $emailFix))) {
                $bannedOn = $emailFix;
                break;
                }
            } else {                // User@host
                 if(($notAllowed = (strtolower($banned) == $emailFix))) {
                $bannedOn = $emailFix;
                break;
                }
            }
            } elseif(preg_match('/^\d{1,3}(\\\.\d{1,3}){0,3}(\/\d{1,2})?$/',
                                $banFix)) {
            // IP address
            if($notAllowed = check_ip($banFix, $_SERVER['REMOTE_ADDR'])) {
                $bannedOn = $_SERVER['REMOTE_ADDR'];
                break;
            }

            // If the client is working through a proxy...
            if(!empty($_SERVER['HTTP_X_FORWARDED_FOR'])) {
                if($notAllowed =
                    check_ip($banFix, $_SERVER['HTTP_X_FORWARDED_FOR']))
                {
                $bannedOn = $_SERVER['HTTP_X_FORWARDED_FOR'];
                break;
                }
            }

            } else {                // Must be a host/domain name
                if(($notAllowed = ereg("$banFix$", $remoteHostFix))) {
                    $bannedOn = $remoteHostFix;
                    break;
                }
            }
        }
    }

    if($notAllowed) {
    $errors[] = $spf_lang['banned']." ($bannedOn)";
    if($logOnBan) {
        error_log("[$scriptName] Banned on \"$bannedOn\"", 0);
    }
    }

    return $notAllowed;
}

// Generate informational headers
function generate_additional_headers() {

    global $reportRemoteHost, $reportRemoteAddr,
       $reportRemoteUser, $reportRemoteIdent, $reportOrigReferer;

        // Remote host/address/user reporting?
    if($reportRemoteHost && !empty($_SERVER['REMOTE_HOST']))
    $addlHeaders .= "X-Remote-Host: " . $_SERVER['REMOTE_HOST'] . "\n";
    if($reportRemoteAddr) {
    if(!empty($_SERVER['REMOTE_ADDR']))
        $addlHeaders .= "X-Remote-Addr: " . $_SERVER['REMOTE_ADDR'] . "\n";

    // If the client is working through a proxy...
    if(!empty($_SERVER['HTTP_X_FORWARDED_FOR']))
        $addlHeaders .= "X-Http-X-Forwarded-For: " .
                 $_SERVER['HTTP_X_FORWARDED_FOR'] . "\n";
    }
    if($reportRemoteUser && !empty($_SERVER['REMOTE_USER']))
    $addlHeaders .= "X-Remote-User: " . $_SERVER['REMOTE_USER'] . "\n";
    if($reportRemoteIdent && !empty($_SERVER['REMOTE_IDENT']))
    $addlHeaders .= "X-Remote-Ident: " . $_SERVER['REMOTE_IDENT'] . "\n";
    if($reportOrigReferer && !empty($_POST['orig_referer']))
    $addlHeaders .= "X-SPForm-Referer: " . $_POST['orig_referer'] . "\n";

    return $addlHeaders;
}

// Mail advisory to $errorsTo list
function mail_advisory($errors, $spf_lang) {

    global $errorsTo, $addSubjSig;
    $finalSubject = "";

    if(!empty($errorsTo)) {
    if($addSubjSig)
        $finalSubject = "[$dfltSubj] ";
    $finalSubject .= $spf_lang['form-problem'];

    $content = $spf_lang['form-problem-content'];
    $nerrors = count($errors);

    for($i = 0; $i < $nerrors; $i++)
        $content .= "    . " . $errors[$i] . "\n";

    $addlHeaders = generate_additional_headers();

    // MS-Win mail servers want crlf and *don't* want a trailing pair
    // Note: This code in two places!  (The alternatives would've been
    // just as ugly, IMO.)
    if(PHP_OS == "WIN32" || PHP_OS == "WINNT") {
        // It seems we only need do this with the "additional headers,"
        // but we're set up here to easily add other mail() variables.
        foreach (array('addlHeaders') as $foo) {
        $$foo = preg_replace("/\n$/", "", $$foo);
        $$foo = preg_replace("/\n/", "\r\n", $$foo);
        }
    }

    mail($errorsTo, $finalSubject, $content, $addlHeaders);
    }
}

// Okay, here we go...

// Check the referer
if(!check_referer($formProcAllowedReferers, $logOnReferer, $spf_lang)) {
    show_fatal($errors,$spf_lang);
    if($adviseOnReferer == true)
    mail_advisory($errors, $spf_lang);
    exit;
}

// "whoto" to email address hash
$whotos = array();

// Get the list of recipient keys and values
if(($fp = fopen($recipientFile, "r")) == false) {
     die($spf_lang['bad-file'].$recipientFile."\n");
}
while($inString = read_file_line($fp)) {
    list($key, $description, $value) = explode(':', $inString);
    $whotos[trim($key)] = trim($value);
}
fclose($fp);

// Convert "whoto" to recipient.
foreach($whotos as $key => $value) {
    if($_POST['whoto'] == $key) {
    $recipient = $value;
    break;
    }
}

// No valid recipient?  That's an error.
if(empty($recipient))
    $errors[] = "\"" . $_POST['whoto'] . "\"".$spf_lang['bad-destination'];

// Gave an email address?  It's an error if not--unless we allow no
// email address.
if(empty($_POST['email'])) {
    // For compatibility with pre-1.2.3 version config files, if
    // $requireEmail is not set, we default to "true".
    if(!isset($requireEmail) || $requireEmail) {
    $errors[] = $spf_lang['no-email'];
    }
// Valid-looking email address?
} elseif(!preg_match('/^[^\s@]+@[a-z0-9\.-]+?\.[a-z]{2,4}$/i', $_POST['email'])) {
    $errors[] = "\"" . $_POST['email'] .
        "\"".$spf_lang['bad-email'];
}

// Check for banned email addresses and remote hosts
if($banned = check_banlist($logOnBan, $_POST['email'], $banListFile)) {

    if($adviseOnBan)
    mail_advisory($errors, $spf_lang);
    if($warnBanned) {
    show_fatal($errors,$spf_lang);
    exit;
    } else {
    unset($errors);        // Make it look as if all's well
    }
}

// Check for name?
if($requireName && empty($_POST['name']))
     $errors[] = $spf_lang['no-name'];

// Check for subject?
if($requireSubj && empty($_POST['subject']))
     $errors[] = $spf_lang['no-subject'];

// Comments?  What's the point, if not?
if(empty($_POST['comments']))
     $errors[] = $spf_lang['no-comments'];

if($useHiddenField) {

    if(!(empty($_POST['Last__Name']))) {  // probable spammer filled the hidden field
            if ($logOnHidden) {
                    error_log("Hidden field filled");
            }
      if ($warnHiddenField) {
         $errors[] = $spf_lang['last-name']; // potentially confusing phrase for non-English speakers
            } else {
            exit; // die quietly
      }
    }
}

if ($requireKeyboard || $requireMouseOrKeyboard) {  // get number of keys pressed
    $keysUsed = $_POST['keyCount'];
}

if ($requireMouse || $requireMouseOrKeyboard) {   // get mouse movement number
    $mouseMove = $_POST['mouseTravel'];
}

if ($requireMouseOrKeyboard) {
    if (($mouseMove < 5)&&($keysUsed < 5)) {
        error_log("No keyboard or mouse movement");
        exit();
    }

} else {
    if ($requireKeyboard) {     // Make sure the keyboard was used
        if ($keysUsed < 5) {
            error_log("No keyboard use");
            exit();
        }
    }

    if ($requireMouse) {   //Make sure the mouse was used
        if ($mouseMove < 5) {
            error_log("No mouse movement");
            exit();
        }
    }
}

if($useTimer) {

    if(!(empty($_POST['tmStart']))) {  // start time for form
        $currentTime = time();
        $startTime = $_POST['tmStart'];
        $startTime -= $timerOffset; // deobfuscate time
        $elapsedTime = $currentTime - $startTime;
            if (($elapsedTime < $useTimerMin) || ($elapsedTime > $useTimerMax)) { // Timer violation
                if ($logTimer) {
                    error_log("Timer Violation");
                }
                if ($warnTimer) {
                     $errors[] = $spf_lang['timeout1'];
                     $errors[] = $spf_lang['timeout2'];;
                } else {
                    exit;
                }
            }

    } else {
        $errors[] = $spf_lang['no-time']; // POST is missing start time and $useTimer=true
    }


}


// Check the verification string
if($requireVerify) {

    if($_SESSION['veriword'] !=  $_POST['verify']) {
        if($logOnVerify) {
            error_log("[$scriptName] Verification failed for host " . $_SERVER['REMOTE_ADDR'], 0);
        }
        if($adviseOnVerify) {
            mail_advisory($errors, $spf_lang);
        }
        if ($warnVerify) {
            $errors[] = $spf_lang['bad-verification'];
        } else {
            exit;
        }

    }
}

// if there were errors, print out an error page and bail out
if(count($errors)) {
    show_errors($errors,$spf_lang);
    return;
}


if(!empty($_POST['name']))
    $content = $_POST['name'] .' '.$spf_lang['wrote'];
$content .= preg_replace('/\r/', '', stripslashes($_POST['comments']));

$addlHeaders = empty($_POST['email'])? "" : "From: " . $_POST['email'] . "\n" .
                                            "Reply-To: " . $_POST['email'] . "\n";

$addlHeaders .= generate_additional_headers();

// Additional recipients?
if(!empty($mailAlso))
    $addlHeaders .= "Cc: ${mailAlso}\n";

// Subject line
if(empty($_POST['subject']))
    $finalSubject = $dfltSubj;
else {
    if($addSubjSig)
    $finalSubject = "[$dfltSubj] ";
    $finalSubject .= addcslashes(stripslashes($_POST['subject']), "\x00..\x1f");
}

// MS-Win mail servers want crlf and *don't* want a trailing pair
// Note: This code in two places!  (The alternatives would've been
// just as ugly, IMO.)
if(PHP_OS == "WIN32" || PHP_OS == "WINNT") {
    // It seems we only need do this with the "additional headers,"
    // but we're set up here to easily add other mail() variables.
    foreach (array('addlHeaders') as $foo) {
    $$foo = preg_replace("/\n$/", "", $$foo);
    $$foo = preg_replace("/\n/", "\r\n", $$foo);
    }
}
// check for email injections, logical email address and overlong fields

if (! preg_match("/^[^@\s]+@([-a-z0-9]+\.)+[a-z]{2,}$/i", $recipient, $result)) {echo $spf_lang['bad-recipient']; error_log("Bad email address"); exit;}

if (strlen($recipient) > 65) {echo $spf_lang['bad-recipient-length']; error_log("recipient length > 65"); exit;}

if (strlen($finalSubject) > 100) {echo $spf_lang['bad-subject-length']; error_log("subject length > 100"); exit;}

if(stristr($finalSubject, "Bcc:") or stristr($finalSubject, "cc:") or stristr($finalSubject, "to:")) {echo $spf_lang['illegal-subject']; error_log("cc: bcc: or to: in subject line"); exit;}

if(stristr($content, "Bcc:") or stristr($content, "cc:") or stristr($content, "to:")) {echo $spf_lang['illegal-message']; error_log("cc: bcc: or to: in content"); exit;}

$linkCount = substr_count($content,'http');  // count links in content
if ($linkCount > $maxLinks) { echo $spf_lang['content-links']; error_log("Too many links in content"); exit;}

// Mail it
if(!$banned) {

    $mail = new PHPMailer();
    // ToDo: next line needs testing
    //$mail->SetLanguage($lang,$modx->config['base_path'].$spformPath."lang");
    if ($spfUseSMTP) {
          $mail->IsSMTP();
        $mail->SMTPAuth = true; //Turn on SMTP authentication
        $mail->Host = $spfSMTP_Host;
        $mail->Port = $spfSMTP_Port;
        $mail->Username = $spfSMTP_UserName; //SMTP username
        $mail->Password = $spfSMTP_Password; //SMTP password
    } else { // not using SMTP - use mail()
        $mail->IsMail();
    }
    $mail->CharSet = $modx->config['modx_charset'];
    $mail->AddAddress($recipient);

    $mail->From = $_POST['email'];

    $mail->FromName    = $site_name;
    $mail->Subject    = $finalSubject;
    $mail->Body        = $content;
     /*print_r($addlHeaders);
     echo "<br><br>";
     var_dump($_SERVER);
     exit; */
    $mail->AddCustomHeader($addlHeaders);
    if (!$test_mode) {
        if(!$mail->send()) {
            webAlert($spf_lang['mail-failure'] . $mail->ErrorInfo);
            exit;
        }
    }





}

// Redirect them to a response page
// $spfResponseArg = "";
// if(!empty($_POST['orig_referer'])) { // if it's there, create arg for spfresponse to take user back
//     $spfResponseArg .= "OrigRef=" . urlencode($_POST['orig_referer'].'"'); // needed for makeUrl
// }
// $responseURL = $modx->makeUrl($spfResponseID,'',$spfResponseArg,'full'); // $spfResponseID set in config file

// if(!empty($_POST['orig_referer'])) {
//     $responseURL = substr_replace($responseURL, '', strlen($responseURL)-3);  // strip trailing slash from original referrer string
// }
// if ($spfDebug) {
//     echo "ResponseURL: ".$responseURL.'<br>';
// }
// $modx->sendRedirect($responseURL,1);

// $spfResponseID set in config file
$responseURL = $modx->makeUrl($spfResponseID, '', '', 'full');   // $spfResponseID set in config file
$modx->sendRedirect($responseURL,1);



?>
